/**
 * 
 */
package org.gcube.gcat;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.gcube.common.keycloak.KeycloakClientFactory;
import org.gcube.common.keycloak.model.TokenResponse;
import org.gcube.common.security.providers.SecretManagerProvider;
import org.gcube.common.security.secrets.AccessTokenSecret;
import org.gcube.common.security.secrets.Secret;
import org.junit.AfterClass;
import org.junit.BeforeClass;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class ContextTest {
	
	protected static Properties properties;
	protected static final String PROPERTIES_FILENAME = "token.properties";
	
	public static final String ROOT;
	public static final String VO;
	public static final String VRE;
	
	static {
		properties = new Properties();
		InputStream input = ContextTest.class.getClassLoader().getResourceAsStream(PROPERTIES_FILENAME);
		
		try {
			// load the properties file
			properties.load(input);
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
		
		// DEFAULT_TEST_SCOPE_NAME = "/pred4s/preprod/preVRE";
		// DEFAULT_TEST_SCOPE_NAME = "/gcube/devsec/devVRE";
		
		ROOT = "/gcube";
		VO = ROOT + "/devsec";
		VRE = VO + "/devVRE";
		// VO = ROOT + "/devNext";
		// VRE = VO + "/NextNext";
		
	}
	
	public static void setContextBySecret(Secret secret) throws Exception {
		SecretManagerProvider.instance.set(secret);
	}
	
	public static void setContextByName(String fullContextName) throws Exception {
		Secret secret = getSecretByContextName(fullContextName);
		setContextBySecret(secret);
	}
	
	private static Secret getSecretByContextName(String fullContextName) throws Exception {
		String clientID = ""; 
		String clientSecret = ContextTest.properties.getProperty(fullContextName);
		TokenResponse tokenResponse = KeycloakClientFactory.newInstance().queryUMAToken(fullContextName, clientID, clientSecret, fullContextName, null);
		return new AccessTokenSecret(tokenResponse.getAccessToken());
	}
	
	@BeforeClass
	public static void beforeClass() throws Exception {
		setContextByName(VRE);
	}
	
	@AfterClass
	public static void afterClass() throws Exception {
		SecretManagerProvider.instance.reset();
	}
	
}
